#include <stdio.h>
#include <stdarg.h>
#include "video.h"

#define MAXSTR  80

extern unsigned char normattr, invattr;

struct text_info
{
    unsigned char winleft;
    unsigned char wintop;
    unsigned char winright;
    unsigned char winbottom;
    unsigned char attribute;
    unsigned char curx;
    unsigned char cury;
};

unsigned BaseOfScreen = 0xb800;
unsigned char winlen, winwid;
struct text_info curwin;
char tmpstr[MAXSTR];

char init_video(void)
{
    asm  mov   ah,0fh
    asm  int   10h
    asm  cmp   ah,80
    asm  jb    nomode
    asm  mov   winwid,ah
    asm  cmp   al,07h
    asm  jne   mode_done
    asm  mov   BaseOfScreen,0b000h
mode_done:
    asm  mov   ah,11h
    asm  mov   al,30h
    asm  mov   bh,00h
    asm  int   10h
    asm  cmp   dl,24
    asm  jb    nomode
    asm  mov   winlen,dl
    asm  jmp   check_done
nomode:
    return 0;
check_done:
    curwin.winleft = 1;
    curwin.wintop = 1;
    curwin.winright = winwid;
    curwin.winbottom = winlen+1;
    curwin.attribute = normattr;
    curwin.curx = 1;
    curwin.cury = 1;
    return 1;
}

void clrscr(void)
{
    scroll(UP, curwin.winleft-1, curwin.wintop-1, curwin.winright-1,
           curwin.winbottom-1, curwin.attribute, 0);
    gotoxy(1, 1);
}

int getkey(void)
{
    asm  mov   ah,00h
    asm  int   16h
    return (_AL ? _AL : _AH+256);
}

void hidecursor(void)
{
    asm  mov   ah,01h
    asm  mov   ch,20h
    asm  int   10h
}

void showcursor(void)
{
    asm  mov   ah,01h
    asm  cmp   BaseOfScreen,0b000h
    asm  je    monocursor
    asm  mov   ch,6
    asm  mov   cl,7
    asm  jmp   showint
monocursor:
    asm  mov   ch,11
    asm  mov   cl,12
showint:
    asm  int   10h
}

void gotoxy(unsigned char xpos, unsigned char ypos)
{
    asm  mov   ah,02h
    asm  mov   bh,0
    _DH = curwin.wintop+ypos-2;
    _DL = curwin.winleft+xpos-2;
    asm  int   10h
    curwin.curx = xpos;
    curwin.cury = ypos;
}

void cputs(char far *str)
{
    asm  push  ds
    asm  push  si
    asm  mov   si,WORD PTR str
    asm  mov   ax,WORD PTR str+2
    asm  mov   ds,ax
    _BH = curwin.attribute;
    _AH = curwin.wintop+(curwin.cury-1);
    _AL = curwin.winleft+(curwin.curx-1);
    DWriteStr();
    asm  pop   si
    asm  pop   ds
}

void gettext(unsigned char left, unsigned char top, unsigned char right,
             unsigned char bottom, void far *destin)
{
    asm  push  es
    asm  push  di
    asm  mov   di,WORD PTR destin
    asm  mov   ax,WORD PTR destin+2
    asm  mov   es,ax
    _BH = bottom-top+1;
    _BL = right-left+1;
    _AH = top;
    _AL = left;
    StoreToMem();
    asm  pop   di
    asm  pop   es
}

void puttext(unsigned char left, unsigned char top, unsigned char right,
             unsigned char bottom, void far *source)
{
    asm  push  si
    asm  mov   si,WORD PTR source
    asm  mov   dx,WORD PTR source+2
    _BH = bottom-top+1;
    _BL = right-left+1;
    _AH = top;
    _AL = left;
    StoreToScr();
    asm  pop   si
}

void cprintf(char *format, ...)
{
    va_list aptr;

    va_start(aptr, format);
    vsprintf(tmpstr, format, aptr);
    cputs(tmpstr);
    va_end(aptr);
}

void textattr(unsigned char attr)
{
    curwin.attribute = attr;
}

void window(unsigned char left, unsigned char top, unsigned char right,
            unsigned char bottom)
{
    curwin.winleft = left;
    curwin.wintop = top;
    curwin.winright = right;
    curwin.winbottom = bottom;
    curwin.curx = 1;
    curwin.cury = 1;
}

void hputchar(unsigned char column, unsigned char row, unsigned char attr,
              unsigned char character, unsigned num)
{
    asm  mov   cx,num
    asm  mov   bh,attr
    asm  mov   bl,character
    asm  mov   ah,row
    asm  mov   al,column
    DWriteCH();
}

void vputchar(unsigned char column, unsigned char row, unsigned char attr,
              unsigned char character, unsigned num)
{
    asm  mov   cx,num
    asm  mov   bh,attr
    asm  mov   bl,character
    asm  mov   ah,row
    asm  mov   al,column
    DWriteCHVert();
}

void framewin(unsigned char attr)
{
    unsigned char x1, y1, x2, y2;

    x1 = curwin.winleft-1;
    y1 = curwin.wintop-1;
    x2 = curwin.winright+1;
    y2 = curwin.winbottom+1;
    hputchar(x1, y1, attr, 213, 1);
    hputchar(x2, y1, attr, 184, 1);
    hputchar(x1, y2, attr, 212, 1);
    hputchar(x2, y2, attr, 190, 1);
    hputchar(x1+1, y1, attr, 205, (x2-x1-1));
    hputchar(x1+1, y2, attr, 205, (x2-x1-1));
    vputchar(x1, y1+1, attr, 179, (y2-y1-1));
    vputchar(x2, y1+1, attr, 179, (y2-y1-1));
}

unsigned char wherey(void)
{
    asm  mov   ah,03h
    asm  mov   bh,0
    asm  int   10h
    return (_DH-curwin.wintop+2);
}

void wtitle(char far *str)
{
    asm  push  ds
    asm  push  si
    asm  mov   si,WORD PTR str
    asm  mov   ax,WORD PTR str+2
    asm  mov   ds,ax
    _BH = invattr;
    _AH = curwin.wintop-1;
    _AL = curwin.winleft+1;
    DWriteStr();
    asm  pop   si
    asm  pop   ds
}

void scroll(unsigned char dir, unsigned char x1, unsigned char y1,
            unsigned char x2, unsigned char y2, unsigned char attr,
            unsigned char num)
{
    _AH = dir;
    _AL = num;
    _BH = attr;
    _CH = y1;
    _CL = x1;
    _DH = y2;
    _DL = x2;
    asm  int   10h
}
